/*
    KTextDecode
    Version 3.0, 21 November 2001
    (C) 2002 Zavolzhsky Alexandr <zavolzhsky@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
*/

#include <kfiledialog.h>
#include <kaboutapplication.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <qaccel.h>
#include <qfileinfo.h>
#include <qtooltip.h>
#include "ktextdecode.h"

KTextDecode::KTextDecode(const char *name) : KMainWindow(0L, name)
{
    setMinimumSize(400,160);
    setMaximumSize(400,160);
    
    btSrc = new QPushButton(i18n("&Source"),this);
    btSrc->setGeometry(5,5,100,20);
    edSrc = new QLineEdit(this);
    edSrc->setGeometry(110,5,280,20);
    QToolTip::add(edSrc,i18n("Source filename"));
    
    btDest = new QPushButton(i18n("&Destination"),this);
    btDest->setGeometry(5,30,100,20);
    edDest = new QLineEdit(this);
    edDest->setGeometry(110,30,280,20);
    QToolTip::add(edDest,i18n("Destination filename"));

    bgType = new QButtonGroup(i18n("Decode type"),this,"decodetype");
    bgType->setGeometry(5,55,280,80);
    rbAnsi2Koi = new QRadioButton(i18n("Windows to Unix"),bgType,"ansi2koi");
    rbAnsi2Koi->setGeometry(7,17,125,15);
    rbKoi2Ansi = new QRadioButton(i18n("Unix to Windows"),bgType,"koi2ansi");
    rbKoi2Ansi->setGeometry(142,17,125,15);
    rbOem2Koi = new QRadioButton(i18n("Dos to Unix"),bgType,"oem2koi");
    rbOem2Koi->setGeometry(7,37,125,15);
    rbKoi2Oem = new QRadioButton(i18n("Unix to Dos"),bgType,"koi2oem");
    rbKoi2Oem->setGeometry(142,37,125,15);
    rbIso2Koi = new QRadioButton(i18n("ISO to Unix "),bgType,"iso2koi");
    rbIso2Koi->setGeometry(7,57,125,15);
    rbKoi2Iso = new QRadioButton(i18n("Unix to ISO"),bgType,"koi2iso");
    rbKoi2Iso->setGeometry(142,57,125,15);
    rbAnsi2Koi->setChecked(true);

    pbDecode = new KProgress(this,"pbDecode");
    pbDecode->setGeometry(5,140,280,15);

    btConvert = new QPushButton(i18n("D&ecode"),this);
    btConvert->setGeometry(290,60,100,24);
    btAbout = new QPushButton(i18n("&About"),this);
    btAbout->setGeometry(290,96,100,24);
    btQuit = new QPushButton(i18n("&Quit"),this);
    btQuit->setGeometry(290,131,100,24);
        
    QAccel *skey = new QAccel(this);
    skey->connectItem(skey->insertItem(ALT+Key_S),this,SLOT(setSrcFile()));
    skey->connectItem(skey->insertItem(ALT+Key_D),this,SLOT(setDestFile()));
    skey->connectItem(skey->insertItem(ALT+Key_C),this,SLOT(decode()));
    skey->connectItem(skey->insertItem(ALT+Key_A),this,SLOT(about()));
    skey->connectItem(skey->insertItem(ALT+Key_Q),this,SLOT(quit()));

    connect(btSrc,SIGNAL(clicked()),this,SLOT(setSrcFile()));
    connect(btDest,SIGNAL(clicked()),this,SLOT(setDestFile()));
    connect(btConvert,SIGNAL(clicked()),this,SLOT(decode()));
    connect(btAbout,SIGNAL(clicked()),this,SLOT(about()));
    connect(btQuit,SIGNAL(clicked()),this,SLOT(quit()));
}

void KTextDecode::setSrcFile()
{
    src = KFileDialog::getOpenFileName();
    if (!src.isEmpty())
    {
        edSrc->setText(src);
        edDest->setText(src + ".new");
    }
}

void KTextDecode::setDestFile()
{
    dest = KFileDialog::getSaveFileName();
    if (!dest.isEmpty()) edDest->setText(dest);
}

bool KTextDecode::isBoxesEmpty()
{
    src = edSrc->text();
    dest = edDest->text();
    
    if (src.isEmpty() || dest.isEmpty())
        return (false);
    else
        return (true);
}

bool KTextDecode::isBoxesSame()
{
    if (strcmp(src,dest) == 0)
        return true;
    else
        return false;
}

void KTextDecode::decode()
{
    if (isBoxesEmpty() && !isBoxesSame())
    {
        if (rbAnsi2Koi->isChecked())
            checkDecode(decodeFile(src,dest,1));
        if (rbKoi2Ansi->isChecked())
            checkDecode(decodeFile(src,dest,2));
        if (rbOem2Koi->isChecked())
            checkDecode(decodeFile(src,dest,3));
        if (rbKoi2Oem->isChecked())
            checkDecode(decodeFile(src,dest,4));
        if (rbIso2Koi->isChecked())
            checkDecode(decodeFile(src,dest,5));
        if (rbKoi2Iso->isChecked())
            checkDecode(decodeFile(src,dest,6));
    }
    else
    {
        KMessageBox::information(this,
                     i18n("Source and destination cannot be empty or the same."),
                     i18n("Warning"));
    }
}

void KTextDecode::checkDecode(bool result)
{
    if (result)
        KMessageBox::information(this,
                     i18n("Decode complete."),
                     i18n("Information"));
    else
        KMessageBox::error(this,
                     i18n("Cannot open source file."),
                     i18n("Error"));
}

bool KTextDecode::decodeFile(QString szIn,QString szOut,int decodeType)
{
    uchar ak[] = {225,226,247,231,228,229,246,250,233,234,235,236,237,238,239,240,
                  242,243,244,245,230,232,227,254,251,253,255,249,248,252,224,241,
                  193,194,215,199,196,197,214,218,201,202,203,204,205,206,207,208,
                  136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,
                  152,153,154,129,156,157,158,159,160,161,162,163,164,165,166,167,
                  168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,
                  210,211,212,213,198,200,195,222,219,221,223,217,216,220,192,209,
                  130,132,135,134,128,133,131,155,184,185,186,187,188,189,190,191};

    uchar ik[] = {128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
                  144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
                  160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,
                  225,226,247,231,228,229,246,250,233,234,235,236,237,238,239,240,
                  242,243,244,245,230,232,227,254,251,253,255,249,248,252,224,241,
                  193,194,215,199,196,197,214,218,201,202,203,204,205,206,207,208,
                  210,211,212,213,198,200,195,222,219,221,223,217,216,220,192,209,
                  176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191};

    uchar ka[] = {244,195,240,246,241,245,243,242,176,177,178,179,180,181,182,183,
                  184,185,186,187,188,189,190,191,192,193,194,247,196,197,198,199,
                  200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,
                  216,217,218,219,220,221,222,223,248,249,250,251,252,253,254,255,
                  238,160,161,230,164,165,228,163,229,168,169,170,171,172,173,174,
                  175,239,224,225,226,227,166,162,236,235,167,232,237,233,231,234,
                  158,128,129,150,132,133,148,131,149,136,137,138,139,140,141,142,
                  143,159,144,145,146,147,134,130,156,155,135,152,157,153,151,154};

    uchar ki[] = {128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
                  144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
                  160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,
                  240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,
                  238,208,209,230,212,213,228,211,229,216,217,218,219,220,221,222,
                  223,239,224,225,226,227,214,210,236,235,215,232,237,233,231,234,
                  206,176,177,198,180,181,196,179,197,184,185,186,187,188,189,190,
                  191,207,192,193,194,195,182,178,204,203,183,200,205,201,199,202};

    uchar kw[] = {128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
                  144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
                  160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,
                  176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,
                  254,224,225,246,228,229,244,227,245,232,233,234,235,236,237,238,
                  239,255,240,241,242,243,230,226,252,251,231,248,253,249,247,250,
                  222,192,193,214,196,197,212,195,213,200,201,202,203,204,205,206,
                  207,223,208,209,210,211,198,194,220,219,199,216,221,217,215,218};

    uchar wk[] = {128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
                  144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
                  160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,
                  176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,
                  225,226,247,231,228,229,246,250,233,234,235,236,237,238,239,240,
                  242,243,244,245,230,232,227,254,251,253,255,249,248,252,224,241,
                  193,194,215,199,196,197,214,218,201,202,203,204,205,206,207,208,
                  210,211,212,213,198,200,195,222,219,221,223,217,216,220,192,209};

    int iCh,iStep,iCount = 0;
    uchar *table = NULL;
    FILE *fIn = NULL,*fOut = NULL;

    if (decodeType == 1) table=wk;
    if (decodeType == 2) table=kw;
    if (decodeType == 3) table=ak;
    if (decodeType == 4) table=ka;
    if (decodeType == 5) table=ik;
    if (decodeType == 6) table=ki;

    fIn = fopen(szIn,"r");
    if (fIn == NULL) return false;
    fOut = fopen(szOut,"w");
    if (fOut == NULL) return false;

    QFileInfo fInfo(szIn);
    iStep = fInfo.size()/100;
    KApplication::setOverrideCursor(waitCursor);

    while((iCh=fgetc(fIn))!=-1)
    {
        iCount++;
        if (iCount >= iStep)
        {
            iCount = 0;
            pbDecode->setValue(pbDecode->value()+1);
        }
        iCh &= 0377;
        if (iCh & 0200) iCh=table[iCh & 0177];
        fputc(iCh,fOut);
    }

    fclose(fOut);
    fclose(fIn);
    pbDecode->setValue(0);
    KApplication::restoreOverrideCursor();
    return true;
}

void KTextDecode::about()
{
    KAboutApplication *ap = new KAboutApplication(this,"ktextdecode-about");
    ap->exec();
}

void KTextDecode::quit()
{
    kapp->quit();
}
